// set the dimensions and margins of the graph
const width = 800;
const height = 600;
const margin = 60;

// append the svg object to the body of the page
const svg = d3
  .select('#my_dataviz')
  .append('svg')
  .attr('width', width / 2)
  .attr('height', height);

const svg2 = d3
  .select('#my_dataviz')
  .append('svg')
  .attr('width', width / 2)
  .attr('height', height);

function render(mapData, countryData, riderData) {
  var projection = d3
    .geoEqualEarth()
    .fitSize([width / 2, height], mapData);
  var path = d3.geoPath(projection);

  //const mapColorAttr = 'age';
  var mapColorScale = d3
    .scaleLinear()
    .domain(
      d3.extent(
        d3.map(
          countryData.values(),
          (r) => r.length, //+r[mapColorAttr], //
        ),
      ),
    )
    .range(['white', 'blue']);
  console.log(mapColorScale.domain());

  function getCountryColor(name) {
    let entry = countryData.get(name);
    if (!(entry == undefined)) {
      return mapColorScale(entry.length);
    }
    return 'grey';
  }

  svg
    .append('g')
    .selectAll('path')
    .data(mapData.features)
    .join('path')
    // draw each country
    .attr('fill', function (d) {
      return getCountryColor(d.properties.name);
    })
    .on('mouseover', function (event, data) {
      //console.log(data);
      //d3.select(this).attr('fill', 'red');
      d3.selectAll(`#${this.id}`)
        .attr('fill', 'red')
        .attr('r', 20);
    })
    .on('mouseleave', function (event, data) {
      d3.selectAll(`#${this.id}.country`).attr(
        'fill',
        getCountryColor(data.properties.name),
      );
      d3.selectAll(`#${this.id}.circle`)
        .attr('fill', getPointColor(data.properties.name))
        .attr('r', 5); //(d) => rScale(d[rAttr]));
    })
    .attr('id', (d) => d.properties.name)
    .attr('d', path)
    .attr('class', 'country');

  const xAttr = 'Year';
  const x = d3
    .scaleLinear()
    .domain(
      d3.extent(
        d3.map(riderData.values(), (d) => +d[xAttr]),
      ),
    )
    .range([margin, width / 2 - margin]);
  //console.log(x.domain());

  const yAttr = 'win_gap';
  const y = d3
    .scaleLinear()
    .domain(
      d3.extent(
        d3.map(
          riderData.values().filter((d) => d[yAttr]),
          (d) => d[yAttr],
        ),
      ),
    )
    .range([height - margin, margin]);

  const plotColorAttr = 'Winner';
  const plotColorScale = d3.scaleOrdinal(
    d3.schemeCategory10,
  );
  function getPointColor(d) {
    return plotColorScale(d);
  }

  /*
  const rAttr = 'Population';
  const rScale = d3
    .scaleLinear()
    .domain(
      d3.extent(
        d3.map(countryData.values(), (d) => d[rAttr]),
      ),
    )
    .range([4, 15]);
  */
  svg2
    .append('g')
    .attr('transform', `translate(0,${height - margin})`) // This controls the vertical position of the Axis
    .call(d3.axisBottom(x));

  svg2
    .append('g')
    .attr('transform', `translate(${margin},0)`) // This controls the vertical position of the Axis
    .call(d3.axisLeft(y));

  svg2
    .append('g')
    .selectAll('circle')
    .data(riderData.values())
    .join('circle')
    .attr('cx', (d) => x(d[xAttr]))
    .attr('cy', (d) => y(d[yAttr]))
    .attr('r', 5) //(d) => rScale(d[rAttr]))
    .attr('fill', (d) => getPointColor(d.Country))
    .attr('id', (d) => d.Country)
    .attr('class', 'circle')
    .on('mouseover', function (event, data) {
      //d3.select(this).attr('fill', 'red');
      d3.selectAll(`#${this.id}.country`)
        .attr('fill', 'red')
        .attr('r', 20)
        .selectAll('circle');
      d3.select(this).attr('fill', 'red').attr('r', 20);
    })
    .on('mouseleave', function (event, data) {
      d3.selectAll(`#${this.id}.country`)
        .attr('fill', getCountryColor(data.Country))
        .attr('r', 5);
      d3.select(this)
        .attr('fill', getPointColor(data.Country))
        .attr('r', 5);
    });
}

//Read the data
const mapUrl =
  'https://raw.githubusercontent.com/holtzy/D3-graph-gallery/master/DATA/world.geojson';
const riderStatsUrl =
  'https://raw.githubusercontent.com/georgeb3ck/dataviz/main/cleanedTDF.csv';

function normalizeCountryName(s) {
  return s.replaceAll(' ', '_').toLowerCase();
}

Promise.all([d3.json(mapUrl), d3.csv(riderStatsUrl)]).then(
  ([mapData, riderData, raceData, palmaresData]) => {
    var countryData = new Map();
    riderData.forEach((row) => {
      row.Country = normalizeCountryName(row.Country);

      if (!countryData.has(row.Country)) {
        countryData.set(row.Country, []);
      }
      countryData.get(row.Country).push(row);

      //countryData.set(row.Country, row);
    });
    mapData.features.forEach((f) => {
      f.properties.name = normalizeCountryName(
        f.properties.name,
      );
    });
    console.log(riderData);
    console.log(countryData);
    render(mapData, countryData, riderData);
  },
);
